<?php

defined ('_JEXEC') or die('Restricted access');

/**
 * Plugin providing Custom variables for VM Shipping by Rules
 *
 * @subpackage Plugins - VmShipmentRules
 * @copyright Copyright (C) 2017 Reinhold Kainhofer, office@open-tools.net
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.txt
 * VirtueMart is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 *
 * http://open-tools.net/
 *
 */
if (!class_exists ('VmPlugin')) {
	require(JPATH_VM_PLUGINS . DS . 'vmplugin.php');
}

/** Extension plugin for the "Shipping by Rules" shipping plugin for VirtueMart
 */
class plgVmShipmentRulesVMCouponPrice extends VmPlugin {
	/**  Trigger to add variables to the cart values
	  *  You can add new variables to the $cartvals array or modify existing ones. They will be directly 
	  *  available in all rules.
	  *  This trigger will be first called right before any rule is evaluated. In that case, $products 
	  *  will contain all products in the cart and $cart_prices will be an arrow containing the calculated
	  *  prices of the order.
	  *  Please notice that this function might also be called for only a subset of products of the cart
	  *  when the plugin evaluates a scoping function like evaluate_for_categories(...).
	  */
	function onVmShippingRulesGetCartValues(&$cartvals, $cart, $products, $method) {
		// The coupon code is in general not yet set (will be set by the cart controller AFTER shipping is handled! 
		// -> set the coupon code now from the coupon_code REQUEST variable.
		// If the coupon is not changed, the coupon variables are already properly set in the cartPrices
		$coupon_code = trim(vRequest::getString('coupon_code', ''));
		if(!empty($coupon_code)){
			$this->couponHandler(/*$cart->couponCode*/$coupon_code, $cart);
		}
		$cartvals['salespriceaftercoupon']= $cart->cartPrices['salesPrice'] + (isset($cart->cartPrices['salesPriceCoupon'])?$cart->cartPrices['salesPriceCoupon']:0);
		$cartvals['amountaftercoupon']= $cart->cartPrices['salesPrice'] + (isset($cart->cartPrices['salesPriceCoupon'])?$cart->cartPrices['salesPriceCoupon']:0);
		$cartvals['couponvalue'] = isset($cart->cartPrices['couponValue'])?$cart->cartPrices['couponValue']:0;
		
	}
	
	# As the coupon handler in calculationh.php is protected, we need to 
	# duplicate all code here instead of calling the VM coupon handler directly....
	protected function couponHandler($_code, &$cart) {
		JPluginHelper::importPlugin('vmcoupon');
		$dispatcher = JDispatcher::getInstance();
		$returnValues = $dispatcher->trigger('plgVmCouponHandler', array($cart->couponCode, &$cart->cartData, &$cart->cartPrices));
		if(!empty($returnValues)){
			foreach ($returnValues as $returnValue) {
				if ($returnValue !== null  ) {
					return $returnValue;
				}
			}
		}

		if (!class_exists('CouponHelper'))
			require(VMPATH_SITE . DS . 'helpers' . DS . 'coupon.php');
		if (!($_data = CouponHelper::getCouponDetails($_code))) {
			return; // TODO give some error here
		}

		$_value_is_total = ($_data->percent_or_total == 'total');

		$cart->cartPrices['salesPriceCoupon'] = ($_value_is_total ? $_data->coupon_value * -1 : ($cart->cartPrices['salesPrice'] * ($_data->coupon_value / 100)) * -1);
		$cart->cartPrices['couponTax'] = 0;
		$cart->cartPrices['couponValue'] = $cart->cartPrices['salesPriceCoupon'] - $cart->cartPrices['couponTax'];

	}
	
}

// No closing tag
